@@ -1,9 +1,11 @@
-use crate::r2_api::{Instruction, Syscall};
+use crate::r2_api::{Instruction, Syscall, hex_decode};
 use crate::value::Value;
-use crate::operations::{Operations, pop_value, 
+use crate::operations::{Operations, pop_value, push_value,
     pop_stack_value, pop_concrete, do_operation, OPS};
 
-use crate::state::{State, StateStatus, StackItem, ExecMode};
+use crate::state::{State, StateStatus, StackItem, 
+    ExecMode, Event, EventTrigger, EventContext, DO_EVENT_HOOKS};
+
 use crate::sims::{SimMethod};
 use crate::sims::syscall::syscall;
 
@@ -16,6 +18,7 @@ const INSTR_NUM: usize = 64;
 const COLOR: bool = false;
 const CALL_TYPE: i64 = 3;
 const RETN_TYPE: i64 = 5;
+// const NOP_TYPE: i64 = 8;
 
 #[derive(Debug, Clone, PartialEq)]
 pub enum Word {
@@ -31,6 +34,7 @@ pub type HookMethod = fn (&mut State) -> bool;
 pub struct Processor {
     pub instructions: HashMap<u64, InstructionEntry>,
     pub hooks: HashMap<u64, Vec<HookMethod>>,
+    pub esil_hooks: HashMap<u64, Vec<String>>,
     pub sims: HashMap<u64, SimMethod>,
     pub traps: HashMap<u64, SimMethod>, 
     pub syscalls: HashMap<u64, Syscall>,
@@ -50,6 +54,7 @@ pub struct Processor {
 pub enum InstructionStatus {
     None,
     Hook,
+    ESILHook,
     Sim,
     Merge,
     Avoid,
@@ -78,6 +83,7 @@ impl Processor {
         Processor {
             instructions: HashMap::new(),
             hooks:        HashMap::new(),
+            esil_hooks:   HashMap::new(),
             sims:         HashMap::new(),
             traps:        HashMap::new(),
             syscalls:     HashMap::new(),
@@ -132,6 +138,9 @@ impl Processor {
                 let poke = self.get_operator(&s[l-4..]).unwrap();
                 tokens.push(Word::Operator(Operations::AddressRestore));
                 tokens.push(poke);
+            
+            } else if let Some(values) = state.context.get(s) {
+                tokens.extend(values.iter().map(|x| Word::Literal(x.to_owned())));
             } else {
                 tokens.push(Word::Unknown(String::from(s)));
             }
@@ -143,14 +152,12 @@ impl Processor {
     /// attempt to tokenize word as number literal (eg. 0x8)
     pub fn get_literal(&self, word: &str) -> Option<Word> {        
         if let Ok(i) = word.parse::<u64>() {
-            let val = Value::Concrete(i, 0);
-            Some(Word::Literal(val))
+            Some(Word::Literal(Value::Concrete(i, 0)))
         } else if word.len() > 2 && &word[0..2] == "0x" {
             let val = u64::from_str_radix(&word[2..word.len()], 16).unwrap();
             Some(Word::Literal(Value::Concrete(val, 0)))
         } else if let Ok(i) = word.parse::<i64>() {
-            let val = Value::Concrete(i as u64, 0);
-            Some(Word::Literal(val))
+            Some(Word::Literal(Value::Concrete(i as u64, 0)))
         } else {
             None
         }
@@ -158,15 +165,19 @@ impl Processor {
 
     /// attempt to tokenize word as register (eg. rbx)
     pub fn get_register(&self,  state: &mut State, word: &str) -> Option<Word> {
-        state.registers.get_register(word).map(|reg| Word::Register(reg.index))
+        let name = if let Some(alias) = state.registers.aliases.get(word) {
+            alias.reg.as_str()
+        } else {
+            word
+        };
+        state.registers.get_register(name).map(|reg| Word::Register(reg.index))
     }
 
     /// attempt to tokenize word as operation (eg. +)
     pub fn get_operator(&self, word: &str) -> Option<Word> {
-        let op = Operations::from_string(word);
-        match op {
+        match Operations::from_string(word) {
             Operations::Unknown => None,
-            _ => Some(Word::Operator(op))
+            op => Some(Word::Operator(op))
         }
     }
 
@@ -184,11 +195,11 @@ impl Processor {
     pub fn do_syscall(&self, state: &mut State) {
         let sys_val = state.registers.get_with_alias("SN");
         let sys_num = state.solver.evalcon_to_u64(&sys_val).unwrap();
-        let pc = state.registers.get_pc().as_u64().unwrap();
+        //let pc = state.registers.get_pc().as_u64().unwrap();
 
         if let Some(sys) = self.syscalls.get(&sys_num) {
-            let cc = state.r2api.get_syscall_cc(pc).unwrap();
-            let mut args = vec!();
+            let cc = state.r2api.get_syscall_cc().unwrap();
+            let mut args = Vec::with_capacity(8);
             for arg in cc.args {
                 args.push(state.registers.get(arg.as_str()));
             }
@@ -325,7 +336,9 @@ impl Processor {
                         Operations::GoTo => {
                             let n = pop_concrete(state, false, false);
                             if let Some(_cond) = &state.condition {
-                                panic!("Hit symbolic GOTO");
+                                println!("Hit symbolic GOTO");
+                                state.set_inactive(); // take the easy way out
+                                break;
                                 //cond.assert();
                             }
                             state.esil.mode = ExecMode::Uncon;
@@ -333,19 +346,20 @@ impl Processor {
                         },
                         Operations::Break => {
                             if let Some(_cond) = &state.condition {
-                                panic!("Hit symbolic BREAK");
+                                println!("Hit symbolic BREAK");
+                                state.set_inactive();
                                 //cond.assert();
                             }
                             break;
                         },
                         Operations::Trap => {
                             let trap = pop_concrete(state, false, false);
-                            let pc = state.registers.get_pc().as_u64().unwrap();
+                            //let pc = state.registers.get_pc().as_u64().unwrap();
 
                             let sys_val = state.registers.get_with_alias("SN");                            
                             if let Some(trap_sim) = self.traps.get(&trap) {
                                 // provide syscall args
-                                let cc = state.r2api.get_syscall_cc(pc).unwrap();
+                                let cc = state.r2api.get_syscall_cc().unwrap();
                                 let mut args = vec!(sys_val);
                                 for arg in cc.args {
                                     args.push(state.registers.get(arg.as_str()));
@@ -359,6 +373,7 @@ impl Processor {
                     }
                 },
                 Word::Unknown(s) => {
+                    push_value(state, Value::Concrete(0, 0));
                     println!("Unknown word: {}", s);
                 }
             }
@@ -536,6 +551,22 @@ impl Processor {
                     self.parse(state, words);
                 }
             },
+            InstructionStatus::ESILHook => {
+                let mut skip = false;
+                let pc_val = Value::Concrete(new_pc, 0);
+                state.registers.set_pc(pc_val);
+
+                let esils = &self.esil_hooks[&pc];
+                for esil in esils {
+                    self.parse_expression(state, esil);
+                    let val = pop_concrete(state, false, false);
+                    skip = (val != 0) || skip;
+                }
+
+                if !skip {
+                    self.parse(state, words);
+                }
+            },
             InstructionStatus::Sim => {
                 let sim = &self.sims[&pc];
                 let pc_val = Value::Concrete(new_pc, 0);
@@ -574,12 +605,23 @@ impl Processor {
     // get the instruction, set its status, tokenize if necessary
     // and optimize if enabled. TODO this has become so convoluted, fix it
     pub fn fetch_instruction(&mut self, state: &mut State, pc_val: u64) {
+        let has_instr = self.instructions.contains_key(&pc_val);
+        if self.selfmodify || !has_instr {
 
-        if self.selfmodify || !self.instructions.contains_key(&pc_val) {
             let mut pc_tmp = pc_val;
             let instrs = if self.selfmodify {
                 let data =  state.memory_read_bytes(pc_val, 32);
                 // 1 at a time for selfmodify
+                // check to see if bytes changed
+                if has_instr {
+                    let instr = &self.instructions[&pc_val];
+                    let bytes = hex_decode(&instr.instruction.bytes);
+                    if data[..bytes.len()] == bytes {
+                        // nothing needs to be done but this is 
+                        // such a weird construction. i hate this code
+                        return; 
+                    }
+                } 
                 state.r2api.disassemble_bytes(pc_val, &data, 1).unwrap()
             } else {
                 state.r2api.disassemble(pc_val, INSTR_NUM).unwrap()
@@ -594,6 +636,8 @@ impl Processor {
                 let mut opt = self.optimized && !self.selfmodify;
                 if self.hooks.contains_key(&pc_tmp) {
                     status = InstructionStatus::Hook;
+                } else if self.esil_hooks.contains_key(&pc_tmp) {
+                    status = InstructionStatus::ESILHook;
                 } else if self.breakpoints.contains_key(&pc_tmp) {
                     status = InstructionStatus::Break;
                 } else if self.mergepoints.contains_key(&pc_tmp) {
@@ -636,6 +680,9 @@ impl Processor {
         if self.debug {
             self.print_instr(state, &instr.instruction);
         }
+        if state.strict && instr.instruction.disasm == "invalid" {
+            panic!("Executed invalid instruction");
+        }
 
         self.execute(state, &instr.instruction, &instr.status, &instr.tokens);
     }
@@ -648,29 +695,42 @@ impl Processor {
         if let Some(pc_val) = pc_value.as_u64() {
             self.execute_instruction(&mut state, pc_val);
         } else {
-            println!("got an unexpected sym PC: {:?}", pc_value);
+            panic!("got an unexpected sym PC: {:?}", pc_value);
         }
 
         let new_pc = state.registers.get_pc();
-        let mut pcs = Vec::with_capacity(pc_allocs);
+        let pcs;
 
         if self.force && !state.esil.pcs.is_empty() {
-            pcs = state.esil.pcs;
-            state.esil.pcs = Vec::with_capacity(pc_allocs);
+            pcs = mem::take(&mut state.esil.pcs);
         } else if let Some(pc) = new_pc.as_u64() {
-                pcs.push(pc);
+            pcs = vec!(pc);
         } else {
             let pc_val = new_pc.as_bv().unwrap();
             if self.debug {
                 println!("\nsymbolic PC: {:?}\n", pc_val);
             }
+
+            if DO_EVENT_HOOKS && state.has_event_hooks {
+                state.do_hooked(&Event::SymbolicExec(EventTrigger::Before), 
+                    &EventContext::ExecContext(new_pc.clone()));
+            }
             
             if self.lazy && !state.esil.pcs.is_empty() {
-                pcs = state.esil.pcs;
-                state.esil.pcs = Vec::with_capacity(pc_allocs);
+                pcs = mem::take(&mut state.esil.pcs);
+            } else if !state.esil.pcs.is_empty() {
+                // testing sat without modelgen is a bit faster than evaluating
+                pcs = mem::take(&mut state.esil.pcs).into_iter()
+                    .filter(|x| state.check(&new_pc.eq(&Value::Concrete(*x, 0))))
+                    .collect();
             } else {
                 pcs = state.evaluate_many(&pc_val);
             }
+
+            if DO_EVENT_HOOKS && state.has_event_hooks {
+                state.do_hooked(&Event::SymbolicExec(EventTrigger::After), 
+                    &EventContext::ExecContext(new_pc.clone()));
+            }
         }
 
         if pcs.len() == 1 && new_pc.as_u64().is_some() {
@@ -682,7 +742,6 @@ impl Processor {
             for new_pc_val in &pcs[..last] {
                 let mut new_state = state.clone();
                 if let Some(pc_val) = new_pc.as_bv() {
-                    //let pc_bv = new_state.translate(&pc_val).unwrap(); 
                     let a = pc_val._eq(&new_state.bvv(*new_pc_val, pc_val.get_width()));
                     new_state.solver.assert(&a);
                 }
@@ -730,15 +789,7 @@ impl Processor {
 
             match current_state.status {
                 StateStatus::Active | StateStatus::PostMerge => {
-                    let mut new_states = self.step(current_state);
-
-                    // this is a weird compromise, check to for Break states
-                    if let Some((i, _s)) = new_states.iter().enumerate()
-                        .find(|(_i, s)| s.status == StateStatus::Break) {
-                        return VecDeque::from(vec!(new_states.remove(i))); 
-                    }
-
-                    states.extend(new_states);
+                    states.extend(self.step(current_state));
                 },
                 StateStatus::Merge => {
                     self.merge(current_state);
@@ -766,7 +817,7 @@ impl Processor {
         } else {
             let mut merge_state = self.merges.remove(&pc).unwrap();
             let state_asserts = state.solver.assertions.clone();
-            let assertion = state.solver.and_all(&state_asserts).unwrap();
+            let assertion = state.solver.and_all(&state_asserts);
             let asserted = Value::Symbolic(assertion.clone(), 0);
 
             // merge registers 
@@ -796,7 +847,7 @@ impl Processor {
 
             // merge solvers
             let assertions = merge_state.solver.assertions.clone();
-            let current = state.solver.and_all(&assertions).unwrap();
+            let current = state.solver.and_all(&assertions);
             merge_state.solver.reset();
             merge_state.assert(&current.or(&assertion));
             self.merges.insert(pc, merge_state);
