@@ -35,6 +35,20 @@ pub struct CallingConvention {
     pub args: Vec<String>
 }
 
+impl Default for CallingConvention {
+    fn default() -> Self {
+        CallingConvention {
+            ret: String::from("A0"),
+            args: vec!(
+                String::from("A0"),
+                String::from("A1"),
+                String::from("A2"),
+                String::from("A3")
+            )
+        }
+    }
+}
+
 impl Endian {
     pub fn from_string(end: &str) -> Endian {
         match end {
@@ -148,33 +162,37 @@ pub struct BinInfo {
     pub nx: bool
 }
 
-fn binfo() -> BinInfo{
-    BinInfo {
-        arch: "".to_string(),
-        bintype: "".to_string(),
-        bits: 64,
-        canary: false,
-        endian: "little".to_string(),
-        os: "".to_string(),
-        nx: false
+impl Default for BinInfo {
+    fn default() -> Self {
+        BinInfo {
+            arch: "".to_string(),
+            bintype: "".to_string(),
+            bits: 64,
+            canary: false,
+            endian: "little".to_string(),
+            os: "".to_string(),
+            nx: false
+        }
     }
 }
 
-fn core() -> CoreInfo {
-    CoreInfo {
-        file: "".to_string(),
-        size: 0,
-        mode: "".to_string(),
-        format: "".to_string()
+impl Default for CoreInfo {
+    fn default() -> Self {
+        CoreInfo {
+            file: "".to_string(),
+            size: 0,
+            mode: "".to_string(),
+            format: "".to_string()
+        }
     }
 }
 
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, Serialize, Default, Deserialize)]
 pub struct Information {
-    #[serde(default="core")]
+    #[serde(default)]
     pub core: CoreInfo,
 
-    #[serde(default="binfo")]
+    #[serde(default)]
     pub bin: BinInfo
 }
 
@@ -350,6 +368,20 @@ pub struct ClassInfo {
     pub superclass: String
 }
 
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct Reference {
+    pub from: u64,
+    pub r#type: String,
+    pub opcode: String,
+
+    #[serde(default)]
+    pub fcn_addr: u64,
+    pub fcn_name: String,
+
+    #[serde(default)]
+    pub refname: String
+}
+
 pub type R2Result<T> = Result<T, String>;
 pub fn r2_result<T, E>(result: Result<T, E>) -> R2Result<T> {
     if let Ok(res) = result {
@@ -395,7 +427,7 @@ impl R2Api {
             None
         };
 
-        let r2pipe = match (filename, opts) {
+        let r2pipe = match (&filename, opts) {
             (None, None) => R2Pipe::open(),
             (Some(name), _) => R2Pipe::spawn(name, options),
             _ => Err("cannot have options for non-spawned")
@@ -442,11 +474,41 @@ impl R2Api {
         r2_result(serde_json::from_str(json.as_str()))
     }
 
-    pub fn get_syscall_cc(&mut self, pc: u64) -> R2Result<CallingConvention> {
+    /*
+        arch/ABI      arg1  arg2  arg3  arg4  arg5  arg6  arg7  Notes
+    ──────────────────────────────────────────────────────────────
+    alpha         a0    a1    a2    a3    a4    a5    -
+    arc           r0    r1    r2    r3    r4    r5    -
+    arm/OABI      a1    a2    a3    a4    v1    v2    v3
+    arm/EABI      r0    r1    r2    r3    r4    r5    r6
+    arm64         x0    x1    x2    x3    x4    x5    -
+    blackfin      R0    R1    R2    R3    R4    R5    -
+    i386          ebx   ecx   edx   esi   edi   ebp   -
+    ia64          out0  out1  out2  out3  out4  out5  -
+    m68k          d1    d2    d3    d4    d5    a0    -
+    microblaze    r5    r6    r7    r8    r9    r10   -
+    mips/o32      a0    a1    a2    a3    -     -     -     [1]
+    mips/n32,64   a0    a1    a2    a3    a4    a5    -
+    nios2         r4    r5    r6    r7    r8    r9    -
+    parisc        r26   r25   r24   r23   r22   r21   -
+    powerpc       r3    r4    r5    r6    r7    r8    r9
+    riscv         a0    a1    a2    a3    a4    a5    -
+    s390          r2    r3    r4    r5    r6    r7    -
+    s390x         r2    r3    r4    r5    r6    r7    -
+    superh        r4    r5    r6    r7    r0    r1    r2
+    sparc/32      o0    o1    o2    o3    o4    o5    -
+    sparc/64      o0    o1    o2    o3    o4    o5    -
+    tile          R00   R01   R02   R03   R04   R05   -
+    x86-64        rdi   rsi   rdx   r10   r8    r9    -
+    x32           rdi   rsi   rdx   r10   r8    r9    -
+    xtensa        a6    a3    a4    a5    a8    a9    -
+    */
+    
+    pub fn get_syscall_cc(&mut self) -> R2Result<CallingConvention> {
         let bin = self.info.as_ref().unwrap().bin.clone();
         // this sucks, need a central place for arch shit
-        if bin.arch == "x86" && bin.bits == 32 {
-            Ok(CallingConvention {
+        match (bin.arch.as_str(), bin.bits) {
+            ("x86", 32) => Ok(CallingConvention {
                 args: vec!(
                     "ebx".to_string(), 
                     "ecx".to_string(), 
@@ -456,9 +518,80 @@ impl R2Api {
                     "ebp".to_string()
                 ),
                 ret: "eax".to_string()
-            })
-        } else {
-            self.get_cc(pc)
+            }),
+            ("x86", 64) => Ok(CallingConvention {
+                args: vec!(
+                    "rdi".to_string(), 
+                    "rsi".to_string(), 
+                    "rdx".to_string(), 
+                    "r10".to_string(), 
+                    "r8".to_string(), 
+                    "r9".to_string()
+                ),
+                ret: "rax".to_string()
+            }),
+            // 16 is thumb mode, need to handle better
+            ("arm", 16) | ("arm", 32) => Ok(CallingConvention {
+                args: vec!(
+                    "r0".to_string(), 
+                    "r1".to_string(), 
+                    "r2".to_string(), 
+                    "r3".to_string(), 
+                    "r4".to_string(), 
+                    "r5".to_string(),
+                    "r6".to_string()
+                ),
+                ret: "r0".to_string()
+            }),
+            ("arm", 64) => Ok(CallingConvention {
+                args: vec!(
+                    "x0".to_string(), 
+                    "x1".to_string(), 
+                    "x2".to_string(), 
+                    "x3".to_string(), 
+                    "x4".to_string(), 
+                    "x5".to_string(),
+                    "x6".to_string(),
+                    "x7".to_string(),
+                    "x8".to_string() // supposedly xnu/ios can have up 9 args
+                ),
+                ret: "x0".to_string()
+            }),
+            ("riscv", _) | ("mips", _) => Ok(CallingConvention {
+                args: vec!(
+                    "a0".to_string(), 
+                    "a1".to_string(), 
+                    "a2".to_string(), 
+                    "a3".to_string(), 
+                    "a4".to_string(), 
+                    "a5".to_string()
+                ),
+                ret: "a0".to_string()
+            }),
+            ("sparc", _) => Ok(CallingConvention {
+                args: vec!(
+                    "o0".to_string(), 
+                    "o1".to_string(), 
+                    "o2".to_string(), 
+                    "o3".to_string(), 
+                    "o4".to_string(), 
+                    "o5".to_string()
+                ),
+                ret: "o0".to_string()
+            }),
+            ("ppc", _) => Ok(CallingConvention {
+                args: vec!(
+                    "r3".to_string(), 
+                    "r4".to_string(), 
+                    "r5".to_string(),
+                    "r6".to_string(), 
+                    "r7".to_string(), 
+                    "r8".to_string(), 
+                    "r9".to_string()
+                ),
+                ret: "r3".to_string() // TODO errors are in r0
+            }),
+            _ => Err("calling convention not found".to_owned())
         }
     }
 
@@ -496,6 +629,22 @@ impl R2Api {
         r2_result(serde_json::from_str(json.as_str()))
     }
 
+    pub fn get_references(&mut self, addr: u64) -> R2Result<Vec<Reference>> {
+        let json = self.cmd(&format!("axtj {}", addr))?;
+        r2_result(serde_json::from_str(json.as_str()))
+    }
+
+    /*pub fn get_strings(&mut self) -> R2Result<Vec<Reference>> {
+        let json = self.cmd("izzj")?;
+        r2_result(serde_json::from_str(json.as_str()))
+    }*/
+
+    pub fn search_strings(&mut self, string: &str) -> R2Result<Vec<u64>> {
+        let result = self.cmd(&format!("izz~[2]~{}", string))?;
+        Ok(result.trim().split("\n").map(|x| u64::from_str_radix(&x[2..], 16)
+            .unwrap_or_default()).filter(|x| *x != 0).collect())
+    }
+
     pub fn get_blocks(&mut self, addr: u64) -> R2Result<Vec<BasicBlock>> {
         let cmd = format!("af @ {}; afbj @ {}", addr, addr);
         let json = self.cmd(cmd.as_str())?;
@@ -574,19 +723,19 @@ impl R2Api {
     pub fn init_frida(&mut self, addr: u64) -> R2Result<HashMap<String, String>> {
         // we are reaching levels of jankiness previously thought to be impossible
         let alloc = self.cmd(":dma 4096").unwrap();
-
-        let func = format!("{{ptr('{}').writeUtf8String(JSON.stringify(this.context))}}",
+        let func = format!(
+            "{{ptr('{}').writeUtf8String(JSON.stringify(this.context))}}",
             alloc.trim());
 
-        let script_data = format!(": Interceptor.attach(ptr('0x{:x}'),function(){});:db {}", 
+        let script_data = format!(
+            ": Interceptor.attach(ptr('0x{:x}'),function(){});:db {}",
             addr, func, addr);
 
         self.cmd(&script_data).unwrap();
-
-        loop { 
+        loop {
             thread::sleep(time::Duration::from_millis(100));
-            let out = self.cmd(&format!("psz 4096 @ {}", alloc))?;
-            if out.contains("{") {
+            if self.cmd(&format!("ps 1 @ {}", alloc))?.trim() == "{" {
+                let out = self.cmd(&format!("psz 4096 @ {}", alloc))?;
                 self.cmd(&format!(":dma- {}", alloc)).unwrap();
                 break r2_result(serde_json::from_str(&out));
             }
@@ -715,9 +864,6 @@ impl R2Api {
             for path in &paths {
                 let lib_path = path.to_owned() + lib;
                 let loaded = full_paths.iter().any(|x| x == &lib_path);
-
-                //println!("{}", lib_path);
-
                 if !loaded && Path::new(&lib_path).exists() {
                     let load_addr = (high_addr & 0xfffffffffffff000) + 0x3000; // idk
                     self.cmd(format!("o {} {}", &lib_path, load_addr).as_str())?;
@@ -733,7 +879,6 @@ impl R2Api {
                                 reloc.vaddr).as_str())?;
                         }
                     }
-
                     if let Ok(librs) = self.load_library_helper(lib_paths, &full_paths) {
                         full_paths = librs;
                     }
