@@ -2,7 +2,7 @@ use crate::r2_api::{R2Api, R2Result, FunctionInfo, BasicBlock, Instruction, Info
 use crate::processor::{Processor, HookMethod};
 use crate::state::{State, StateStatus};
 //use crate::value::Value;
-use crate::sims::{get_sims, SimMethod, unconstrained};
+use crate::sims::{get_sims, SimMethod, zero};
 use crate::sims::syscall::{indirect};
 use crate::value::Value;
 
@@ -25,6 +25,8 @@ pub enum RadiusOption {
     Optimize(bool),
     /// Enable debug output
     Debug(bool),
+    /// panic! on unimplemented 
+    Strict(bool),
     /// Don't check sat on symbolic pcs
     Lazy(bool),
     /// Check memory permissions
@@ -63,10 +65,11 @@ pub enum RadiusOption {
 pub struct Radius {
     pub r2api: R2Api,
     pub processor: Processor,
-    pub processors: Arc<Mutex<Vec<Processor>>>,
+    processors: Arc<Mutex<Vec<Processor>>>,
     pub eval_max: usize,
     pub check: bool,
-    pub debug: bool
+    pub debug: bool,
+    pub strict: bool
 }
 
 impl Radius {
@@ -100,7 +103,7 @@ impl Radius {
     pub fn new_with_options<T: AsRef<str>>(
         filename: Option<T>, 
         options: &[RadiusOption]) -> Self {
-                
+
         let mut argv = vec!();
         let mut eval_max = 256;
         let mut paths = vec!();
@@ -117,9 +120,7 @@ impl Radius {
         let debug = options.contains(&RadiusOption::Debug(true));
         let use_sims = !options.contains(&RadiusOption::Sims(false));
 
-        if !debug {
-            argv.push("-2"); // silence r2 errors
-        } else {
+        if debug {
             // pretty print disasm + esil
             argv.push("-e scr.color=3");
             argv.push("-e asm.cmt.esil=true");
@@ -127,6 +128,8 @@ impl Radius {
             argv.push("-e asm.emu=false");
         }
 
+        argv.push("-2");
+
         // need this for sims
         if use_sims {
             argv.push("-e io.cache=true");
@@ -148,6 +151,7 @@ impl Radius {
         let check = options.contains(&RadiusOption::Permissions(true));
         let sim_all = options.contains(&RadiusOption::SimAll(true));
         let selfmod = options.contains(&RadiusOption::SelfModify(true));
+        let strict = options.contains(&RadiusOption::Strict(true));
         let mut processor = Processor::new(selfmod, opt, debug, lazy, force, topological);
         let processors = Arc::new(Mutex::new(vec!()));
 
@@ -179,7 +183,8 @@ impl Radius {
             processors,
             eval_max,
             check,
-            debug
+            debug,
+            strict
         }
     }
 
@@ -231,7 +236,7 @@ impl Radius {
      * let mut state = radius.entry_state(&["r100"], &[]);
      * ```
      */
-    pub fn entry_state<T: AsRef<str>>(&mut self, args: &[T], env: &[T]) -> State {
+    pub fn entry_state(&mut self) -> State {
         // get the entrypoint
         let entry = self.r2api.get_entrypoints().unwrap()[0].vaddr;
         self.r2api.seek(entry);
@@ -239,10 +244,11 @@ impl Radius {
         let mut state = self.init_state();
         state.memory.add_stack();
         state.memory.add_heap();
+        state.memory.add_std_streams();
 
         // we write args to both regs and stack
         // i think this is ok
-        let sp = state.registers.get_with_alias("SP");
+        /*let sp = state.registers.get_with_alias("SP");
         let ptrlen = (state.memory.bits/8) as usize;
         let argc = Value::Concrete(args.len() as u64, 0);
         state.memory_write_value(
@@ -288,7 +294,7 @@ impl Radius {
                 &current, &Value::Concrete(0, 0), ptrlen); // write a long null
                 
             current = current + Value::Concrete(ptrlen as u64, 0); 
-        }
+        }*/
 
         let start_main_reloc = self.r2api.get_address(
             "reloc.__libc_start_main").unwrap();
@@ -299,12 +305,49 @@ impl Radius {
         state
     }
 
+    /// Set argv and env with values instead of the dumb string way
+    pub fn set_argv_env(&mut self, state: &mut State, args: &[Value], env: &[Value]) {
+
+        // we write args to both regs and stack
+        // i think this is ok
+        let sp = state.registers.get_with_alias("SP");
+        let ptrlen = (state.memory.bits/8) as usize;
+        let argc = Value::Concrete(args.len() as u64, 0);
+        state.memory_write_value(&sp, &argc, ptrlen);
+        state.registers.set_with_alias("A0", argc);
+
+        let types = ["argv", "env"];
+        let mut current = sp+Value::Concrete(ptrlen as u64, 0);
+        for (i, strings) in [args, env].iter().enumerate() {
+            state.context.insert(types[i].to_owned(), vec!(current.clone()));
+            let alias = format!("A{}", i+1);
+            state.registers.set_with_alias(&alias, current.clone());
+            for string in strings.iter() {
+                let addr = state.memory.alloc(
+                    &Value::Concrete((string.size()/8) as u64 +1, 0));
+
+                state.memory_write_value(
+                    &Value::Concrete(addr, 0), string, string.size() as usize/8);
+
+                state.memory.write_value(
+                    addr+(string.size()/8) as u64, &Value::Concrete(0,0), 1);
+
+                state.memory_write_value(
+                    &current, &Value::Concrete(addr, 0), ptrlen);
+
+                current = current + Value::Concrete(ptrlen as u64, 0); 
+            }
+            state.memory_write_value(&current, &Value::Concrete(0, 0), ptrlen); 
+            current = current + Value::Concrete(ptrlen as u64, 0); 
+        }
+    }
+
     pub fn init_state(&mut self) -> State {
-        State::new(&mut self.r2api, self.eval_max, self.debug, false, self.check)
+        State::new(&mut self.r2api, self.eval_max, self.debug, false, self.check, self.strict)
     }
 
     pub fn blank_state(&mut self) -> State {
-        State::new(&mut self.r2api, self.eval_max, self.debug, true, self.check)
+        State::new(&mut self.r2api, self.eval_max, self.debug, true, self.check, self.strict)
     }
 
     /// Blank except for PC and SP
@@ -317,19 +360,29 @@ impl Radius {
         state.registers.set_with_alias("SP", Value::Concrete(sp, 0));
         state.memory.add_stack();
         state.memory.add_heap();
-        //state.memory.add_std_streams();
+        state.memory.add_std_streams();
         state
     }
 
     /// Hook an address with a callback that is passed the `State`
     pub fn hook(&mut self, addr: u64, hook_callback: HookMethod) {
-        let hooks = self.processor.hooks.remove(&addr);
-        if let Some(mut hook_vec) = hooks {
-            hook_vec.push(hook_callback);
-            self.processor.hooks.insert(addr, hook_vec);
-        } else {
-            self.processor.hooks.insert(addr, vec!(hook_callback));
-        }
+        let mut hooks = self.processor.hooks.remove(&addr).unwrap_or_default();
+        hooks.push(hook_callback);
+        self.processor.hooks.insert(addr, hooks);
+    }
+
+    /// Hook an address with an esil expression
+    pub fn esil_hook(&mut self, addr: u64, esil: &str) {
+        let mut esils = self.processor.esil_hooks.remove(&addr).unwrap_or_default();
+        esils.push(esil.to_owned());
+        self.processor.esil_hooks.insert(addr, esils);
+    }
+
+        
+    /// Hook a symbol with a callback that is passed each state that reaches it
+    pub fn hook_symbol(&mut self, sym: &str, hook_callback: HookMethod) {
+        let addr = self.get_address(sym).unwrap();
+        self.hook(addr, hook_callback);
     }
 
     // internal method to register import sims 
@@ -357,7 +410,7 @@ impl Radius {
             if sim_all {
                 for addr in symmap.values() {
                     // we are gonna go with unconstrained by default
-                    processor.sims.insert(*addr, unconstrained);
+                    processor.sims.insert(*addr, zero);
                 }
             }
         }
@@ -410,6 +463,10 @@ impl Radius {
      * 
      */
     pub fn run(&mut self, state: State, mut threads: usize) -> Option<State> {
+        if threads == 1 {
+            return self.processor.run(state, false).pop_front();
+        }
+
         let mut handles = Vec::with_capacity(threads);
         let statevector = Arc::new(Mutex::new(VecDeque::with_capacity(self.eval_max)));
         statevector.lock().unwrap().push_back(state);
